@@ -31,8 +31,7 @@ import sys
 import os
 
 def is_stdlib(p):
-    return ((p.startswith(sys.prefix)
-            or p.startswith(sys.base_prefix))
+    return ((p.startswith((sys.prefix, sys.base_prefix)))
             and 'site-packages' not in p)
 
 stdlib = {p for p in sys.path if is_stdlib(p)}
