@@ -1,33 +1,42 @@
 """Tests for OO layer of several polynomial representations. """
 
+from sympy.functions.elementary.miscellaneous import sqrt
 from sympy.polys.domains import ZZ, QQ
 from sympy.polys.polyclasses import DMP, DMF, ANP
-from sympy.polys.polyerrors import ExactQuotientFailed, NotInvertible
+from sympy.polys.polyerrors import (CoercionFailed, ExactQuotientFailed,
+                                    NotInvertible)
 from sympy.polys.specialpolys import f_polys
-from sympy.testing.pytest import raises
+from sympy.testing.pytest import raises, warns_deprecated_sympy
 
 f_0, f_1, f_2, f_3, f_4, f_5, f_6 = [ f.to_dense() for f in f_polys() ]
 
 def test_DMP___init__():
-    f = DMP([[0], [], [0, 1, 2], [3]], ZZ)
+    f = DMP([[ZZ(0)], [], [ZZ(0), ZZ(1), ZZ(2)], [ZZ(3)]], ZZ)
 
-    assert f.rep == [[1, 2], [3]]
+    assert f._rep == [[1, 2], [3]]
     assert f.dom == ZZ
     assert f.lev == 1
 
-    f = DMP([[1, 2], [3]], ZZ, 1)
+    f = DMP([[ZZ(1), ZZ(2)], [ZZ(3)]], ZZ, 1)
 
-    assert f.rep == [[1, 2], [3]]
+    assert f._rep == [[1, 2], [3]]
     assert f.dom == ZZ
     assert f.lev == 1
 
-    f = DMP({(1, 1): 1, (0, 0): 2}, ZZ, 1)
+    f = DMP.from_dict({(1, 1): ZZ(1), (0, 0): ZZ(2)}, 1, ZZ)
 
-    assert f.rep == [[1, 0], [2]]
+    assert f._rep == [[1, 0], [2]]
     assert f.dom == ZZ
     assert f.lev == 1
 
 
+def test_DMP_rep_deprecation():
+    f = DMP([1, 2, 3], ZZ)
+
+    with warns_deprecated_sympy():
+        assert f.rep == [1, 2, 3]
+
+
 def test_DMP___eq__():
     assert DMP([[ZZ(1), ZZ(2)], [ZZ(3)]], ZZ) == \
         DMP([[ZZ(1), ZZ(2)], [ZZ(3)]], ZZ)
@@ -43,11 +52,11 @@ def test_DMP___eq__():
 
 def test_DMP___bool__():
     assert bool(DMP([[]], ZZ)) is False
-    assert bool(DMP([[1]], ZZ)) is True
+    assert bool(DMP([[ZZ(1)]], ZZ)) is True
 
 
 def test_DMP_to_dict():
-    f = DMP([[3], [], [2], [], [8]], ZZ)
+    f = DMP([[ZZ(3)], [], [ZZ(2)], [], [ZZ(8)]], ZZ)
 
     assert f.to_dict() == \
         {(4, 0): 3, (2, 0): 2, (0, 0): 8}
@@ -58,34 +67,34 @@ def test_DMP_to_dict():
 
 def test_DMP_properties():
     assert DMP([[]], ZZ).is_zero is True
-    assert DMP([[1]], ZZ).is_zero is False
+    assert DMP([[ZZ(1)]], ZZ).is_zero is False
 
-    assert DMP([[1]], ZZ).is_one is True
-    assert DMP([[2]], ZZ).is_one is False
+    assert DMP([[ZZ(1)]], ZZ).is_one is True
+    assert DMP([[ZZ(2)]], ZZ).is_one is False
 
-    assert DMP([[1]], ZZ).is_ground is True
-    assert DMP([[1], [2], [1]], ZZ).is_ground is False
+    assert DMP([[ZZ(1)]], ZZ).is_ground is True
+    assert DMP([[ZZ(1)], [ZZ(2)], [ZZ(1)]], ZZ).is_ground is False
 
-    assert DMP([[1], [2, 0], [1, 0]], ZZ).is_sqf is True
-    assert DMP([[1], [2, 0], [1, 0, 0]], ZZ).is_sqf is False
+    assert DMP([[ZZ(1)], [ZZ(2), ZZ(0)], [ZZ(1), ZZ(0)]], ZZ).is_sqf is True
+    assert DMP([[ZZ(1)], [ZZ(2), ZZ(0)], [ZZ(1), ZZ(0), ZZ(0)]], ZZ).is_sqf is False
 
-    assert DMP([[1, 2], [3]], ZZ).is_monic is True
-    assert DMP([[2, 2], [3]], ZZ).is_monic is False
+    assert DMP([[ZZ(1), ZZ(2)], [ZZ(3)]], ZZ).is_monic is True
+    assert DMP([[ZZ(2), ZZ(2)], [ZZ(3)]], ZZ).is_monic is False
 
-    assert DMP([[1, 2], [3]], ZZ).is_primitive is True
-    assert DMP([[2, 4], [6]], ZZ).is_primitive is False
+    assert DMP([[ZZ(1), ZZ(2)], [ZZ(3)]], ZZ).is_primitive is True
+    assert DMP([[ZZ(2), ZZ(4)], [ZZ(6)]], ZZ).is_primitive is False
 
 
 def test_DMP_arithmetics():
-    f = DMP([[2], [2, 0]], ZZ)
+    f = DMP([[ZZ(2)], [ZZ(2), ZZ(0)]], ZZ)
 
-    assert f.mul_ground(2) == DMP([[4], [4, 0]], ZZ)
-    assert f.quo_ground(2) == DMP([[1], [1, 0]], ZZ)
+    assert f.mul_ground(2) == DMP([[ZZ(4)], [ZZ(4), ZZ(0)]], ZZ)
+    assert f.quo_ground(2) == DMP([[ZZ(1)], [ZZ(1), ZZ(0)]], ZZ)
 
     raises(ExactQuotientFailed, lambda: f.exquo_ground(3))
 
-    f = DMP([[-5]], ZZ)
-    g = DMP([[5]], ZZ)
+    f = DMP([[ZZ(-5)]], ZZ)
+    g = DMP([[ZZ(5)]], ZZ)
 
     assert f.abs() == g
     assert abs(f) == g
@@ -101,7 +110,7 @@ def test_DMP_arithmetics():
     assert f + 5 == h
     assert 5 + f == h
 
-    h = DMP([[-10]], ZZ)
+    h = DMP([[ZZ(-10)]], ZZ)
 
     assert f.sub(g) == h
     assert f - g == h
@@ -109,7 +118,7 @@ def test_DMP_arithmetics():
     assert f - 5 == h
     assert 5 - f == -h
 
-    h = DMP([[-25]], ZZ)
+    h = DMP([[ZZ(-25)]], ZZ)
 
     assert f.mul(g) == h
     assert f * g == h
@@ -117,7 +126,7 @@ def test_DMP_arithmetics():
     assert f * 5 == h
     assert 5 * f == h
 
-    h = DMP([[25]], ZZ)
+    h = DMP([[ZZ(25)]], ZZ)
 
     assert f.sqr() == h
     assert f.pow(2) == h
@@ -125,11 +134,11 @@ def test_DMP_arithmetics():
 
     raises(TypeError, lambda: f.pow('x'))
 
-    f = DMP([[1], [], [1, 0, 0]], ZZ)
-    g = DMP([[2], [-2, 0]], ZZ)
+    f = DMP([[ZZ(1)], [], [ZZ(1), ZZ(0), ZZ(0)]], ZZ)
+    g = DMP([[ZZ(2)], [ZZ(-2), ZZ(0)]], ZZ)
 
-    q = DMP([[2], [2, 0]], ZZ)
-    r = DMP([[8, 0, 0]], ZZ)
+    q = DMP([[ZZ(2)], [ZZ(2), ZZ(0)]], ZZ)
+    r = DMP([[ZZ(8), ZZ(0), ZZ(0)]], ZZ)
 
     assert f.pdiv(g) == (q, r)
     assert f.pquo(g) == q
@@ -137,11 +146,11 @@ def test_DMP_arithmetics():
 
     raises(ExactQuotientFailed, lambda: f.pexquo(g))
 
-    f = DMP([[1], [], [1, 0, 0]], ZZ)
-    g = DMP([[1], [-1, 0]], ZZ)
+    f = DMP([[ZZ(1)], [], [ZZ(1), ZZ(0), ZZ(0)]], ZZ)
+    g = DMP([[ZZ(1)], [ZZ(-1), ZZ(0)]], ZZ)
 
-    q = DMP([[1], [1, 0]], ZZ)
-    r = DMP([[2, 0, 0]], ZZ)
+    q = DMP([[ZZ(1)], [ZZ(1), ZZ(0)]], ZZ)
+    r = DMP([[ZZ(2), ZZ(0), ZZ(0)]], ZZ)
 
     assert f.div(g) == (q, r)
     assert f.quo(g) == q
@@ -153,11 +162,35 @@ def test_DMP_arithmetics():
 
     raises(ExactQuotientFailed, lambda: f.exquo(g))
 
+    f = DMP([ZZ(1), ZZ(0), ZZ(-1)], ZZ)
+    g = DMP([ZZ(2), ZZ(-2)], ZZ)
+
+    q = DMP([], ZZ)
+    r = f
+
+    pq = DMP([ZZ(2), ZZ(2)], ZZ)
+    pr = DMP([], ZZ)
+
+    assert f.div(g) == (q, r)
+    assert f.quo(g) == q
+    assert f.rem(g) == r
+
+    assert divmod(f, g) == (q, r)
+    assert f // g == q
+    assert f % g == r
+
+    raises(ExactQuotientFailed, lambda: f.exquo(g))
+
+    assert f.pdiv(g) == (pq, pr)
+    assert f.pquo(g) == pq
+    assert f.prem(g) == pr
+    assert f.pexquo(g) == pq
+
 
 def test_DMP_functionality():
-    f = DMP([[1], [2, 0], [1, 0, 0]], ZZ)
-    g = DMP([[1], [1, 0]], ZZ)
-    h = DMP([[1]], ZZ)
+    f = DMP([[ZZ(1)], [ZZ(2), ZZ(0)], [ZZ(1), ZZ(0), ZZ(0)]], ZZ)
+    g = DMP([[ZZ(1)], [ZZ(1), ZZ(0)]], ZZ)
+    h = DMP([[ZZ(1)]], ZZ)
 
     assert f.degree() == 2
     assert f.degree_list() == (2, 2)
@@ -172,15 +205,15 @@ def test_DMP_functionality():
     assert f.max_norm() == 2
     assert f.l1_norm() == 4
 
-    u = DMP([[2], [2, 0]], ZZ)
+    u = DMP([[ZZ(2)], [ZZ(2), ZZ(0)]], ZZ)
 
     assert f.diff(m=1, j=0) == u
     assert f.diff(m=1, j=1) == u
 
     raises(TypeError, lambda: f.diff(m='x', j=0))
 
-    u = DMP([1, 2, 1], ZZ)
-    v = DMP([1, 2, 1], ZZ)
+    u = DMP([ZZ(1), ZZ(2), ZZ(1)], ZZ)
+    v = DMP([ZZ(1), ZZ(2), ZZ(1)], ZZ)
 
     assert f.eval(a=1, j=0) == u
     assert f.eval(a=1, j=1) == v
@@ -199,25 +232,39 @@ def test_DMP_functionality():
     assert (4*f).content() == ZZ(4)
     assert (4*f).primitive() == (ZZ(4), f)
 
-    f = DMP([[1], [2], [3], [4], [5], [6]], ZZ)
+    f = DMP([QQ(1,3), QQ(1)], QQ)
+    g = DMP([QQ(1,7), QQ(1)], QQ)
 
-    assert f.trunc(3) == DMP([[1], [-1], [], [1], [-1], []], ZZ)
+    assert f.cancel(g) == f.cancel(g, include=True) == (
+        DMP([QQ(7), QQ(21)], QQ),
+        DMP([QQ(3), QQ(21)], QQ)
+    )
+    assert f.cancel(g, include=False) == (
+        QQ(7),
+        QQ(3),
+        DMP([QQ(1), QQ(3)], QQ),
+        DMP([QQ(1), QQ(7)], QQ)
+    )
+
+    f = DMP([[ZZ(1)], [ZZ(2)], [ZZ(3)], [ZZ(4)], [ZZ(5)], [ZZ(6)]], ZZ)
+
+    assert f.trunc(3) == DMP([[ZZ(1)], [ZZ(-1)], [], [ZZ(1)], [ZZ(-1)], []], ZZ)
 
     f = DMP(f_4, ZZ)
 
     assert f.sqf_part() == -f
     assert f.sqf_list() == (ZZ(-1), [(-f, 1)])
 
-    f = DMP([[-1], [], [], [5]], ZZ)
-    g = DMP([[3, 1], [], []], ZZ)
-    h = DMP([[45, 30, 5]], ZZ)
+    f = DMP([[ZZ(-1)], [], [], [ZZ(5)]], ZZ)
+    g = DMP([[ZZ(3), ZZ(1)], [], []], ZZ)
+    h = DMP([[ZZ(45), ZZ(30), ZZ(5)]], ZZ)
 
-    r = DMP([675, 675, 225, 25], ZZ)
+    r = DMP([ZZ(675), ZZ(675), ZZ(225), ZZ(25)], ZZ)
 
     assert f.subresultants(g) == [f, g, h]
     assert f.resultant(g) == r
 
-    f = DMP([1, 3, 9, -13], ZZ)
+    f = DMP([ZZ(1), ZZ(3), ZZ(9), ZZ(-13)], ZZ)
 
     assert f.discriminant() == -11664
 
@@ -233,33 +280,34 @@ def test_DMP_functionality():
 
     assert f.invert(g) == s
 
-    f = DMP([[1], [2], [3]], QQ)
+    f = DMP([[QQ(1)], [QQ(2)], [QQ(3)]], QQ)
 
     raises(ValueError, lambda: f.half_gcdex(f))
     raises(ValueError, lambda: f.gcdex(f))
 
     raises(ValueError, lambda: f.invert(f))
 
-    f = DMP([1, 0, 20, 0, 150, 0, 500, 0, 625, -2, 0, -10, 9], ZZ)
-    g = DMP([1, 0, 0, -2, 9], ZZ)
-    h = DMP([1, 0, 5, 0], ZZ)
+    f = DMP(ZZ.map([1, 0, 20, 0, 150, 0, 500, 0, 625, -2, 0, -10, 9]), ZZ)
+    g = DMP([ZZ(1), ZZ(0), ZZ(0), ZZ(-2), ZZ(9)], ZZ)
+    h = DMP([ZZ(1), ZZ(0), ZZ(5), ZZ(0)], ZZ)
 
     assert g.compose(h) == f
     assert f.decompose() == [g, h]
 
-    f = DMP([[1], [2], [3]], QQ)
+    f = DMP([[QQ(1)], [QQ(2)], [QQ(3)]], QQ)
 
     raises(ValueError, lambda: f.decompose())
     raises(ValueError, lambda: f.sturm())
 
 
 def test_DMP_exclude():
-    f = [[[[[[[[[[[[[[[[[[[[[[[[[[1]], [[]]]]]]]]]]]]]]]]]]]]]]]]]]
+    f = [[[[[[[[[[[[[[[[[[[[[[[[[[ZZ(1)]], [[]]]]]]]]]]]]]]]]]]]]]]]]]]
     J = [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17,
         18, 19, 20, 21, 22, 24, 25]
 
-    assert DMP(f, ZZ).exclude() == (J, DMP([1, 0], ZZ))
-    assert DMP([[1], [1, 0]], ZZ).exclude() == ([], DMP([[1], [1, 0]], ZZ))
+    assert DMP(f, ZZ).exclude() == (J, DMP([ZZ(1), ZZ(0)], ZZ))
+    assert DMP([[ZZ(1)], [ZZ(1), ZZ(0)]], ZZ).exclude() ==\
+            ([], DMP([[ZZ(1)], [ZZ(1), ZZ(0)]], ZZ))
 
 
 def test_DMF__init__():
@@ -412,8 +460,8 @@ def test_ANP___init__():
 
     f = ANP(rep, mod, QQ)
 
-    assert f.rep == [QQ(1), QQ(1)]
-    assert f.mod == [QQ(1), QQ(0), QQ(1)]
+    assert f.to_list() == [QQ(1), QQ(1)]
+    assert f.mod_to_list() == [QQ(1), QQ(0), QQ(1)]
     assert f.dom == QQ
 
     rep = {1: QQ(1), 0: QQ(1)}
@@ -421,16 +469,22 @@ def test_ANP___init__():
 
     f = ANP(rep, mod, QQ)
 
-    assert f.rep == [QQ(1), QQ(1)]
-    assert f.mod == [QQ(1), QQ(0), QQ(1)]
+    assert f.to_list() == [QQ(1), QQ(1)]
+    assert f.mod_to_list() == [QQ(1), QQ(0), QQ(1)]
     assert f.dom == QQ
 
     f = ANP(1, mod, QQ)
 
-    assert f.rep == [QQ(1)]
-    assert f.mod == [QQ(1), QQ(0), QQ(1)]
+    assert f.to_list() == [QQ(1)]
+    assert f.mod_to_list() == [QQ(1), QQ(0), QQ(1)]
     assert f.dom == QQ
 
+    f = ANP([1, 0.5], mod, QQ)
+
+    assert all(QQ.of_type(a) for a in f.to_list())
+
+    raises(CoercionFailed, lambda: ANP([sqrt(2)], mod, QQ))
+
 
 def test_ANP___eq__():
     a = ANP([QQ(1), QQ(1)], [QQ(1), QQ(0), QQ(1)], QQ)
@@ -517,26 +571,18 @@ def test_ANP_arithmetics():
     assert q == a/b # == c
 
 def test_ANP_unify():
-    mod = [QQ(1), QQ(0), QQ(-2)]
+    mod_z = [ZZ(1), ZZ(0), ZZ(-2)]
+    mod_q = [QQ(1), QQ(0), QQ(-2)]
 
-    a = ANP([QQ(1)], mod, QQ)
-    b = ANP([ZZ(1)], mod, ZZ)
+    a = ANP([QQ(1)], mod_q, QQ)
+    b = ANP([ZZ(1)], mod_z, ZZ)
 
     assert a.unify(b)[0] == QQ
     assert b.unify(a)[0] == QQ
     assert a.unify(a)[0] == QQ
     assert b.unify(b)[0] == ZZ
 
-
-def test___hash__():
-    # issue 5571
-    # Make sure int vs. long doesn't affect hashing with Python ground types
-    assert DMP([[1, 2], [3]], ZZ) == DMP([[int(1), int(2)], [int(3)]], ZZ)
-    assert hash(DMP([[1, 2], [3]], ZZ)) == hash(DMP([[int(1), int(2)], [int(3)]], ZZ))
-    assert DMF(
-        ([[1, 2], [3]], [[1]]), ZZ) == DMF(([[int(1), int(2)], [int(3)]], [[int(1)]]), ZZ)
-    assert hash(DMF(([[1, 2], [3]], [[1]]), ZZ)) == hash(DMF(([[int(1),
-                int(2)], [int(3)]], [[int(1)]]), ZZ))
-    assert ANP([1, 1], [1, 0, 1], ZZ) == ANP([int(1), int(1)], [int(1), int(0), int(1)], ZZ)
-    assert hash(
-        ANP([1, 1], [1, 0, 1], ZZ)) == hash(ANP([int(1), int(1)], [int(1), int(0), int(1)], ZZ))
+    assert a.unify_ANP(b)[-1] == QQ
+    assert b.unify_ANP(a)[-1] == QQ
+    assert a.unify_ANP(a)[-1] == QQ
+    assert b.unify_ANP(b)[-1] == ZZ
